@@ -40,7 +40,7 @@
 #define WINHEIGHT 200
 
 /* Define this to use triple-buffering in first method. */
-/* #define TRIPLEBUFFERING */
+#define TRIPLEBUFFERING
 
 
 unsigned char *vbuf;
@@ -279,7 +279,7 @@ void demo3(void)
 }
 
 
-void main(void)
+int main(void)
 {
     vga_init();
 
