@@ -56,7 +56,7 @@ static devfs_handle_t devfs_handle;
 #endif
 
 #if defined(KERNEL_2_6) && !defined(CONFIG_DEVFS_FS)
-struct class_simple *svgalib_helper_class;
+struct K_CLASS *svgalib_helper_class;
 #endif
 
 static int check_io_range(int port, int device) {
