@@ -112,9 +112,9 @@ static void neo_setpage(int page)
 #ifdef DEBUG
   fprintf(stderr, "neo_setpage: %d\n", page);
 #endif
-  outb(GRAX, 0x11);
-  outw(GRAX, ((port_in(GRAX+1) & 0xfc) << 8) | 0x11);
-  outw(GRAX, (page << 10) | 0x15); /* set read/write bank */
+  port_out_r(GRAX, 0x11);
+  port_outw_r(GRAX, ((port_in(GRAX+1) & 0xfc) << 8) | 0x11);
+  port_outw_r(GRAX, (page << 10) | 0x15); /* set read/write bank */
 }
 
 static void neo_setrdpage(int page)
@@ -122,9 +122,9 @@ static void neo_setrdpage(int page)
 #ifdef DEBUG
   fprintf(stderr, "neo_setrdpage: %d\n", page);
 #endif
-  outb(GRAX, 0x11);
-  outw(GRAX, (((port_in(GRAX+1) & 0xfc) | 0x01) << 8) | 0x11);
-  outw(GRAX, (page << 10) | 0x15); /* set read bank */
+  port_out_r(GRAX, 0x11);
+  port_outw_r(GRAX, (((port_in(GRAX+1) & 0xfc) | 0x01) << 8) | 0x11);
+  port_outw_r(GRAX, (page << 10) | 0x15); /* set read bank */
 }
 
 static void neo_setwrpage(int page)
@@ -132,9 +132,9 @@ static void neo_setwrpage(int page)
 #ifdef DEBUG
   fprintf(stderr, "neo_setwrpage: %d\n", page);
 #endif
-  outb(GRAX, 0x11);
-  outw(GRAX, (((port_in(GRAX+1) & 0xfc) | 0x01) << 8) | 0x11);
-  outw(GRAX, (page << 10) | 0x16); /* set write bank */
+  port_out_r(GRAX, 0x11);
+  port_outw_r(GRAX, (((port_in(GRAX+1) & 0xfc) | 0x01) << 8) | 0x11);
+  port_outw_r(GRAX, (page << 10) | 0x16); /* set write bank */
 }
 
 static int __svgalib_neo_inlinearmode(void)
@@ -183,62 +183,62 @@ static int neo_saveregs(unsigned char regs[])
   fprintf(stderr, "neo_saveregs\n");
 #endif
   neo_unlock();
-  outw(GRAX, 0x0015);   /* bank#0 */
+  port_outw_r(GRAX, 0x0015);   /* bank#0 */
 
-  outb(GRAX, 0x0a); regs[GeneralLockReg] = port_in(GRAX + 1);
-  outb(GRAX, 0x0e); regs[ExtCRTDispAddr] = port_in(GRAX + 1);
-  outb(GRAX, 0x0f); regs[ExtCRTOffset] = port_in(GRAX + 1);
-  outb(GRAX, 0x10); regs[SysIfaceCntl1] = port_in(GRAX + 1);
-  outb(GRAX, 0x11); regs[SysIfaceCntl2] = port_in(GRAX + 1);
-  outb(GRAX, 0x15); regs[SingleAddrPage] = port_in(GRAX + 1);
-  outb(GRAX, 0x16); regs[DualAddrPage] = port_in(GRAX+1);
-  outb(GRAX, 0x20); regs[PanelDispCntlReg1] = port_in(GRAX+1);
-  outb(GRAX, 0x25); regs[PanelDispCntlReg2] = port_in(GRAX+1);
-  outb(GRAX, 0x30); regs[PanelDispCntlReg3] = port_in(GRAX+1);
-  outb(GRAX, 0x28); regs[PanelVertCenterReg1] = port_in(GRAX+1);
-  outb(GRAX, 0x29); regs[PanelVertCenterReg2] = port_in(GRAX+1);
-  outb(GRAX, 0x2a); regs[PanelVertCenterReg3] = port_in(GRAX+1);
+  port_out_r(GRAX, 0x0a); regs[GeneralLockReg] = port_in(GRAX + 1);
+  port_out_r(GRAX, 0x0e); regs[ExtCRTDispAddr] = port_in(GRAX + 1);
+  port_out_r(GRAX, 0x0f); regs[ExtCRTOffset] = port_in(GRAX + 1);
+  port_out_r(GRAX, 0x10); regs[SysIfaceCntl1] = port_in(GRAX + 1);
+  port_out_r(GRAX, 0x11); regs[SysIfaceCntl2] = port_in(GRAX + 1);
+  port_out_r(GRAX, 0x15); regs[SingleAddrPage] = port_in(GRAX + 1);
+  port_out_r(GRAX, 0x16); regs[DualAddrPage] = port_in(GRAX+1);
+  port_out_r(GRAX, 0x20); regs[PanelDispCntlReg1] = port_in(GRAX+1);
+  port_out_r(GRAX, 0x25); regs[PanelDispCntlReg2] = port_in(GRAX+1);
+  port_out_r(GRAX, 0x30); regs[PanelDispCntlReg3] = port_in(GRAX+1);
+  port_out_r(GRAX, 0x28); regs[PanelVertCenterReg1] = port_in(GRAX+1);
+  port_out_r(GRAX, 0x29); regs[PanelVertCenterReg2] = port_in(GRAX+1);
+  port_out_r(GRAX, 0x2a); regs[PanelVertCenterReg3] = port_in(GRAX+1);
   if (NeoChipset != PCI_CHIP_NM2070){
-    outb(GRAX, 0x32); regs[PanelVertCenterReg4] = port_in(GRAX+1);
-    outb(GRAX, 0x33); regs[PanelHorizCenterReg1] = port_in(GRAX+1);
-    outb(GRAX, 0x34); regs[PanelHorizCenterReg2] = port_in(GRAX+1);
-    outb(GRAX, 0x35); regs[PanelHorizCenterReg3] = port_in(GRAX+1);
+    port_out_r(GRAX, 0x32); regs[PanelVertCenterReg4] = port_in(GRAX+1);
+    port_out_r(GRAX, 0x33); regs[PanelHorizCenterReg1] = port_in(GRAX+1);
+    port_out_r(GRAX, 0x34); regs[PanelHorizCenterReg2] = port_in(GRAX+1);
+    port_out_r(GRAX, 0x35); regs[PanelHorizCenterReg3] = port_in(GRAX+1);
   }
   if (NeoChipset == PCI_CHIP_NM2160){
-    outb(GRAX, 0x36); regs[PanelHorizCenterReg4] = port_in(GRAX+1);
+    port_out_r(GRAX, 0x36); regs[PanelHorizCenterReg4] = port_in(GRAX+1);
   }
   if (NeoChipset == PCI_CHIP_NM2200){
-    outb(GRAX, 0x36); regs[PanelHorizCenterReg4] = port_in(GRAX+1);
-    outb(GRAX, 0x37); regs[PanelVertCenterReg5]  = port_in(GRAX+1);
-    outb(GRAX, 0x38); regs[PanelHorizCenterReg5] = port_in(GRAX+1);
+    port_out_r(GRAX, 0x36); regs[PanelHorizCenterReg4] = port_in(GRAX+1);
+    port_out_r(GRAX, 0x37); regs[PanelVertCenterReg5]  = port_in(GRAX+1);
+    port_out_r(GRAX, 0x38); regs[PanelHorizCenterReg5] = port_in(GRAX+1);
   }
-  outb(GRAX, 0x90); regs[ExtColorModeSelect] = port_in(GRAX+1);
-  outb(GRAX, 0x9B); regs[VCLK3NumeratorLow]  = port_in(GRAX+1);
+  port_out_r(GRAX, 0x90); regs[ExtColorModeSelect] = port_in(GRAX+1);
+  port_out_r(GRAX, 0x9B); regs[VCLK3NumeratorLow]  = port_in(GRAX+1);
   if (NeoChipset == PCI_CHIP_NM2200){
-    outb(GRAX, 0x8F); regs[VCLK3NumeratorHigh] = port_in(GRAX+1);
+    port_out_r(GRAX, 0x8F); regs[VCLK3NumeratorHigh] = port_in(GRAX+1);
   }
-  outb(GRAX, 0x9F); regs[VCLK3Denominator] = port_in(GRAX+1);
+  port_out_r(GRAX, 0x9F); regs[VCLK3Denominator] = port_in(GRAX+1);
 
   regs[EXT_SAVED] = TRUE;
-  outb(vgaIOBase + 4, 0x25); regs[EXTCR + 0x25] = port_in(vgaIOBase + 5);
-  outb(vgaIOBase + 4, 0x2F); regs[EXTCR + 0x2F] = port_in(vgaIOBase + 5);
+  port_out_r(vgaIOBase + 4, 0x25); regs[EXTCR + 0x25] = port_in(vgaIOBase + 5);
+  port_out_r(vgaIOBase + 4, 0x2F); regs[EXTCR + 0x2F] = port_in(vgaIOBase + 5);
   for (i = 0x40; i <= 0x59; i++){
-    outb(vgaIOBase + 4, i); regs[EXTCR + i] = port_in(vgaIOBase + 5);
+    port_out_r(vgaIOBase + 4, i); regs[EXTCR + i] = port_in(vgaIOBase + 5);
   }
   for (i = 0x60; i <= 0x69; i++){
-    outb(vgaIOBase + 4, i); regs[EXTCR + i] = port_in(vgaIOBase + 5);
+    port_out_r(vgaIOBase + 4, i); regs[EXTCR + i] = port_in(vgaIOBase + 5);
   }
   for (i = 0x70; i <= NEO_EXT_CR_MAX; i++){
-    outb(vgaIOBase + 4, i); regs[EXTCR + i] = port_in(vgaIOBase + 5);
+    port_out_r(vgaIOBase + 4, i); regs[EXTCR + i] = port_in(vgaIOBase + 5);
   }
 
   for (i = 0x0A; i <= NEO_EXT_GR_MAX; i++){
-    outb(GRAX, i); regs[EXTGR + i] = port_in(GRAX+1);
+    port_out_r(GRAX, i); regs[EXTGR + i] = port_in(GRAX+1);
   }
 
   /* DAC */
-  outb(0x3C6,0xFF); /* mask */
-  outb(0x3C7,0x00); /* read address */
+  port_out_r(0x3C6,0xFF); /* mask */
+  port_out_r(0x3C7,0x00); /* read address */
   for (i = 0; i < 768; i++){
     regs[DAC + i] = port_in(0x3C9);
     DACDelay;
@@ -258,12 +258,12 @@ static void neo_setregs(const unsigned char regs[], int mode)
   fprintf(stderr, "neo_setregs\n");
 #endif
   neo_unlock();		
-  outw(GRAX, 0x0015);   /* bank#0 */
+  port_outw_r(GRAX, 0x0015);   /* bank#0 */
 
-  outb(GRAX, 0x0a); outb(GRAX+1, regs[GeneralLockReg]);
+  port_out_r(GRAX, 0x0a); port_out_r(GRAX+1, regs[GeneralLockReg]);
 
   /* set color mode first */
-  outb(GRAX, 0x90); temp = port_in(GRAX+1);
+  port_out_r(GRAX, 0x90); temp = port_in(GRAX+1);
   switch (NeoChipset){
   case PCI_CHIP_NM2070:
     temp &= 0xF0; /* Save bits 7:4 */
@@ -275,36 +275,36 @@ static void neo_setregs(const unsigned char regs[], int mode)
     temp |= (regs[ExtColorModeSelect] & ~0x70);
     break;
   }
-  outb(GRAX, 0x90); outb(GRAX+1, temp);
+  port_out_r(GRAX, 0x90); port_out_r(GRAX+1, temp);
 
   /* Disable horizontal and vertical graphics and text expansions */
-  outb(GRAX, 0x25);
+  port_out_r(GRAX, 0x25);
   temp = port_in(GRAX+1);
-  outb(GRAX, 0x25);
+  port_out_r(GRAX, 0x25);
   temp &= 0x39;
-  outb(GRAX+1, temp);
+  port_out_r(GRAX+1, temp);
 
   usleep(200000);
 
   /* DAC */
-  outb(0x3C6,0xFF); /* mask */
-  outb(0x3C8,0x00); /* write address */
+  port_out_r(0x3C6,0xFF); /* mask */
+  port_out_r(0x3C8,0x00); /* write address */
   for (i = 0; i < 768; i++){
-    outb(0x3C9, regs[DAC + i]);
+    port_out_r(0x3C9, regs[DAC + i]);
     DACDelay;
   }
 
-  outb(GRAX, 0x0E); outb(GRAX+1, regs[ExtCRTDispAddr]);
-  outb(GRAX, 0x0F); outb(GRAX+1, regs[ExtCRTOffset]);
-  outb(GRAX, 0x10); temp = port_in(GRAX+1);
+  port_out_r(GRAX, 0x0E); port_out_r(GRAX+1, regs[ExtCRTDispAddr]);
+  port_out_r(GRAX, 0x0F); port_out_r(GRAX+1, regs[ExtCRTOffset]);
+  port_out_r(GRAX, 0x10); temp = port_in(GRAX+1);
   temp &= 0x0F; /* Save bits 3:0 */
   temp |= (regs[SysIfaceCntl1] & ~0x0F);
-  outb(GRAX, 0x10); outb(GRAX+1, temp);
+  port_out_r(GRAX, 0x10); port_out_r(GRAX+1, temp);
 
-  outb(GRAX, 0x11); outb(GRAX+1, regs[SysIfaceCntl2]);
-  outb(GRAX, 0x15); outb(GRAX+1, regs[SingleAddrPage]);
-  outb(GRAX, 0x16); outb(GRAX+1, regs[DualAddrPage]);
-  outb(GRAX, 0x20); temp = port_in(GRAX+1);
+  port_out_r(GRAX, 0x11); port_out_r(GRAX+1, regs[SysIfaceCntl2]);
+  port_out_r(GRAX, 0x15); port_out_r(GRAX+1, regs[SingleAddrPage]);
+  port_out_r(GRAX, 0x16); port_out_r(GRAX+1, regs[DualAddrPage]);
+  port_out_r(GRAX, 0x20); temp = port_in(GRAX+1);
   switch (NeoChipset){
   case PCI_CHIP_NM2070:
     temp &= 0xFC; /* Save bits 7:2 */
@@ -320,75 +320,75 @@ static void neo_setregs(const unsigned char regs[], int mode)
     temp |= (regs[PanelDispCntlReg1] & ~0x98);
     break;
   }
-  outb(GRAX, 0x20); outb(GRAX+1, temp);
-  outb(GRAX, 0x25); temp = port_in(GRAX+1);
+  port_out_r(GRAX, 0x20); port_out_r(GRAX+1, temp);
+  port_out_r(GRAX, 0x25); temp = port_in(GRAX+1);
   temp &= 0x38; /* Save bits 5:3 */
   temp |= (regs[PanelDispCntlReg2] & ~0x38);
-  outb(GRAX, 0x25); outb(GRAX+1, temp);
+  port_out_r(GRAX, 0x25); port_out_r(GRAX+1, temp);
 
   if (NeoChipset != PCI_CHIP_NM2070){
-    outb(GRAX, 0x30); temp = port_in(GRAX+1);
+    port_out_r(GRAX, 0x30); temp = port_in(GRAX+1);
     temp &= 0xEF; /* Save bits 7:5 and bits 3:0 */
     temp |= (regs[PanelDispCntlReg3] & ~0xEF);
-    outb(GRAX, 0x30); outb(GRAX+1, temp);
+    port_out_r(GRAX, 0x30); port_out_r(GRAX+1, temp);
   }
 
-  outb(GRAX, 0x28); outb(GRAX+1, regs[PanelVertCenterReg1]);
-  outb(GRAX, 0x29); outb(GRAX+1, regs[PanelVertCenterReg2]);
-  outb(GRAX, 0x2a); outb(GRAX+1, regs[PanelVertCenterReg3]);
+  port_out_r(GRAX, 0x28); port_out_r(GRAX+1, regs[PanelVertCenterReg1]);
+  port_out_r(GRAX, 0x29); port_out_r(GRAX+1, regs[PanelVertCenterReg2]);
+  port_out_r(GRAX, 0x2a); port_out_r(GRAX+1, regs[PanelVertCenterReg3]);
 
   if (NeoChipset != PCI_CHIP_NM2070){
-    outb(GRAX, 0x32); outb(GRAX+1, regs[PanelVertCenterReg4]);
-    outb(GRAX, 0x33); outb(GRAX+1, regs[PanelHorizCenterReg1]);
-    outb(GRAX, 0x34); outb(GRAX+1, regs[PanelHorizCenterReg2]);
-    outb(GRAX, 0x35); outb(GRAX+1, regs[PanelHorizCenterReg3]);
+    port_out_r(GRAX, 0x32); port_out_r(GRAX+1, regs[PanelVertCenterReg4]);
+    port_out_r(GRAX, 0x33); port_out_r(GRAX+1, regs[PanelHorizCenterReg1]);
+    port_out_r(GRAX, 0x34); port_out_r(GRAX+1, regs[PanelHorizCenterReg2]);
+    port_out_r(GRAX, 0x35); port_out_r(GRAX+1, regs[PanelHorizCenterReg3]);
   }
 
   if (NeoChipset == PCI_CHIP_NM2160){
-    outb(GRAX, 0x36); outb(GRAX+1, regs[PanelHorizCenterReg4]);
+    port_out_r(GRAX, 0x36); port_out_r(GRAX+1, regs[PanelHorizCenterReg4]);
   }
 
   if (NeoChipset == PCI_CHIP_NM2200){
-    outb(GRAX, 0x36); outb(GRAX+1, regs[PanelHorizCenterReg4]);
-    outb(GRAX, 0x37); outb(GRAX+1, regs[PanelVertCenterReg5]);
-    outb(GRAX, 0x38); outb(GRAX+1, regs[PanelHorizCenterReg5]);
+    port_out_r(GRAX, 0x36); port_out_r(GRAX+1, regs[PanelHorizCenterReg4]);
+    port_out_r(GRAX, 0x37); port_out_r(GRAX+1, regs[PanelVertCenterReg5]);
+    port_out_r(GRAX, 0x38); port_out_r(GRAX+1, regs[PanelHorizCenterReg5]);
   }
 
 #if 0
-  outb(GRAX, 0x9B); outb(GRAX+1, regs[VCLK3NumeratorLow]);
+  port_out_r(GRAX, 0x9B); port_out_r(GRAX+1, regs[VCLK3NumeratorLow]);
   if (NeoChipset == PCI_CHIP_NM2200){
-    outb(GRAX, 0x8F); temp = port_in(GRAX+1);
+    port_out_r(GRAX, 0x8F); temp = port_in(GRAX+1);
     temp &= 0x0F; /* Save bits 3:0 */
     temp |= (regs[VCLK3NumeratorHigh] & ~0x0F);
-    outb(GRAX, 0x8F); outb(GRAX+1, temp);
+    port_out_r(GRAX, 0x8F); port_out_r(GRAX+1, temp);
   }
-  outb(GRAX, 0x9F); outb(GRAX+1, regs[VCLK3Denominator]);
+  port_out_r(GRAX, 0x9F); port_out_r(GRAX+1, regs[VCLK3Denominator]);
 #endif
 
   if (regs[EXT_SAVED]){
-    outb(vgaIOBase + 4, 0x25); outb(vgaIOBase + 5, regs[EXTCR + 0x25]);
-    outb(vgaIOBase + 4, 0x2F); outb(vgaIOBase + 5, regs[EXTCR + 0x2F]);
+    port_out_r(vgaIOBase + 4, 0x25); port_out_r(vgaIOBase + 5, regs[EXTCR + 0x25]);
+    port_out_r(vgaIOBase + 4, 0x2F); port_out_r(vgaIOBase + 5, regs[EXTCR + 0x2F]);
     for (i = 0x40; i <= 0x59; i++){
-      outb(vgaIOBase + 4, i); outb(vgaIOBase + 5, regs[EXTCR + i]);
+      port_out_r(vgaIOBase + 4, i); port_out_r(vgaIOBase + 5, regs[EXTCR + i]);
     }
     for (i = 0x60; i <= 0x69; i++){
-      outb(vgaIOBase + 4, i); outb(vgaIOBase + 5, regs[EXTCR + i]);
+      port_out_r(vgaIOBase + 4, i); port_out_r(vgaIOBase + 5, regs[EXTCR + i]);
     }
     for (i = 0x70; i <= NEO_EXT_CR_MAX; i++){
-      outb(vgaIOBase + 4, i); outb(vgaIOBase + 5, regs[EXTCR + i]);
+      port_out_r(vgaIOBase + 4, i); port_out_r(vgaIOBase + 5, regs[EXTCR + i]);
     }
 
     for (i = 0x0a; i <= 0x3f; i++){
-      outb(GRAX, i); outb(GRAX+1, regs[EXTGR + i]);
+      port_out_r(GRAX, i); port_out_r(GRAX+1, regs[EXTGR + i]);
     }
     for (i = 0x90; i <= NEO_EXT_GR_MAX; i++){
-      outb(GRAX, i); outb(GRAX+1, regs[EXTGR + i]);
+      port_out_r(GRAX, i); port_out_r(GRAX+1, regs[EXTGR + i]);
     }
   }
 
   /* Program vertical extension register */
   if (NeoChipset == PCI_CHIP_NM2200){
-    outb(vgaIOBase + 4, 0x70); outb(vgaIOBase + 5, regs[VerticalExt]);
+    port_out_r(vgaIOBase + 4, 0x70); port_out_r(vgaIOBase + 5, regs[VerticalExt]);
   }
 }
 
@@ -703,10 +703,10 @@ static void neo_unlock(void)
 #ifdef DEBUG
     fprintf(stderr, "neo_unlock\n");
 #endif
-    outb(vgaIOBase + 4, 0x11);
+    port_out_r(vgaIOBase + 4, 0x11);
     temp = port_in(vgaIOBase + 5);
-    outb(vgaIOBase + 5, temp & 0x7F);
-    outw(GRAX, 0x2609); /* Unlock NeoMagic registers */
+    port_out_r(vgaIOBase + 5, temp & 0x7F);
+    port_outw_r(GRAX, 0x2609); /* Unlock NeoMagic registers */
 }
 
 static void neo_lock(void)
@@ -716,10 +716,10 @@ static void neo_lock(void)
 #ifdef DEBUG
     fprintf(stderr, "neo_lock\n");
 #endif
-    outb(vgaIOBase + 4, 0x11);
+    port_out_r(vgaIOBase + 4, 0x11);
     temp = port_in(vgaIOBase + 5);
-    outb(vgaIOBase + 5, temp | 0x80);
-    outw(GRAX, 0x0009); /* Lock NeoMagic registers */
+    port_out_r(vgaIOBase + 5, temp | 0x80);
+    port_outw_r(GRAX, 0x0009); /* Lock NeoMagic registers */
 }
 
 
@@ -755,12 +755,12 @@ static void neo_setdisplaystart(int address)
   fprintf(stderr, "neo_setdisplaystart: 0x%x\n", address);
 #endif
   address=address >> 2;
-  outw(vgaIOBase + 4, (address & 0x00FF00) | 0x0C);
-  outw(vgaIOBase + 4, ((address & 0x00FF) << 8) | 0x0D);
+  port_outw_r(vgaIOBase + 4, (address & 0x00FF00) | 0x0C);
+  port_outw_r(vgaIOBase + 4, ((address & 0x00FF) << 8) | 0x0D);
 
-  outb(GRAX, 0x0E);
+  port_out_r(GRAX, 0x0E);
   oldExtCRTDispAddr = port_in(GRAX+1);
-  outw(GRAX,
+  port_outw_r(GRAX,
        ((((address >> 16) & 0x07) | (oldExtCRTDispAddr & 0xf8)) << 8) | 0x0E);
 }
 
@@ -776,8 +776,8 @@ static void neo_setlogicalwidth(int width)
     fprintf(stderr, "neo_setlogicalwidth: %d\n", width);
 #endif
     __svgalib_outCR(0x13,offset&0xff);
-    outb(GRAX, 0x0F); 
-    outb(GRAX+1, width >> 11);
+    port_out_r(GRAX, 0x0F); 
+    port_out_r(GRAX+1, width >> 11);
 }
 
 static int neo_linear(int op, int param)
@@ -923,7 +923,7 @@ static int neo_init(int force, int par1, int par2)
     }
 
     /* Determine panel width -- used in NeoValidMode. */
-    outb(GRAX, 0x20);
+    port_out_r(GRAX, 0x20);
     w = port_in(GRAX+1);
     switch ((w & 0x18) >> 3){
     case 0x00:
