@@ -231,15 +231,15 @@ static void oak_setregs(const unsigned char regs[], int mode)
 	OAKSetByte(OTI87_XREADSEGMENT, regs[OTI87_XREADSEGMENT_DATA]);
 	OAKSetByte(OTI87_XWRITESEGMENT, regs[OTI87_XWRITESEGMENT_DATA]);
 
-	outb(SEQ_I, 0);		/* reset sync. */
+	port_out_r(SEQ_I, 0);		/* reset sync. */
 	junk = port_in(SEQ_D);
-	outw(SEQ_I, 0x00 + ((junk & 0xfd) << 8));
+	port_outw_r(SEQ_I, 0x00 + ((junk & 0xfd) << 8));
 
 	OAKSetByte(OTI87_CLOCK, regs[EXT + 8]);
 	OAKSetByte(OTI87_FIFO, regs[OTI87_FIFO_DATA]);
 	OAKSetByte(OTI87_MODESELECT, regs[OTI87_MODESELECT_DATA]);
 
-	outw(SEQ_I, 0x00 + (junk << 8));	/* set sync. */
+	port_outw_r(SEQ_I, 0x00 + (junk << 8));	/* set sync. */
 
 	OAKSetByte(OTI87_XCRTC, regs[OTI87_XCRTC_DATA]);
 	OAKSetByte(OTI87_OVERFLOW, regs[EXT + 9]);
@@ -249,14 +249,14 @@ static void oak_setregs(const unsigned char regs[], int mode)
 	OAKSetByte(OTI_SEGMENT, regs[OTI_SEGMENT_DATA]);
 
 	/* doc says don't OTI-MISC unless sync. reset is off */
-	outb(SEQ_I, 0);
+	port_out_r(SEQ_I, 0);
 	junk = port_in(SEQ_D);
-	outw(SEQ_I, 0x00 + ((junk & 0xFD) << 8));	/* now disable the timing sequencer */
+	port_outw_r(SEQ_I, 0x00 + ((junk & 0xFD) << 8));	/* now disable the timing sequencer */
 
 	OAKSetByte(OTI_MISC, regs[OTI_MISC_DATA]);
 
 	/* put sequencer back */
-	outw(SEQ_I, 0x00 + (junk << 8));
+	port_outw_r(SEQ_I, 0x00 + (junk << 8));
 
 	OAKSetByte(OTI_BCOMPAT, regs[OTI_BCOMPAT_DATA]);
 	OAKSetByte(OTI_CONFIG, regs[OTI_CONFIG_DATA]);
@@ -391,9 +391,9 @@ static void oak_unlock(void)
 {
     int temp;
     /* Unprotect CRTC[0-7] */
-    outb(__svgalib_CRT_I, 0x11);
+    port_out_r(__svgalib_CRT_I, 0x11);
     temp = port_in(__svgalib_CRT_D);
-    outb(__svgalib_CRT_D, temp & 0x7F);
+    port_out_r(__svgalib_CRT_D, temp & 0x7F);
     temp = OAKGetByte(OTI_CRT_CNTL);
     OAKSetByte(OTI_CRT_CNTL, temp & 0xf0);
     oak_lockreg_save = temp;
@@ -412,9 +412,9 @@ static void oak_lock(void)
     /* don't set the i/o write test bit, though we cleared it on entry */
     OAKSetByte(OTI_CRT_CNTL, oak_lockreg_save & 0xf7);
     /* Protect CRTC[0-7] */
-    outb(__svgalib_CRT_I, 0x11);
+    port_out_r(__svgalib_CRT_I, 0x11);
     temp = port_in(__svgalib_CRT_D);
-    outb(__svgalib_CRT_D, (temp & 0x7F) | 0x80);
+    port_out_r(__svgalib_CRT_D, (temp & 0x7F) | 0x80);
 }
 
 /* oak_test()                                           Probe for Oak card
@@ -508,11 +508,11 @@ static void oak_setwrpage(int page)
 
 static void oak_setdisplaystart(int address)
 {
-    outw(CRT_IC, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
-    outw(CRT_IC, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
+    port_outw_r(CRT_IC, 0x0d + ((address >> 2) & 0x00ff) * 256);	/* sa2-sa9 */
+    port_outw_r(CRT_IC, 0x0c + ((address >> 2) & 0xff00));	/* sa10-sa17 */
     port_in(0x3da);			/* set ATC to addressing mode */
-    outb(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
-    outb(ATT_IW, (port_in(ATT_R) & 0xf0) | ((address & 3) << 1));
+    port_out_r(ATT_IW, 0x13 + 0x20);	/* select ATC reg 0x13 */
+    port_out_r(ATT_IW, (port_in(ATT_R) & 0xf0) | ((address & 3) << 1));
     /* write sa0-1 to bits 1-2 */
 
     if (oak_chiptype == 87) {
@@ -538,7 +538,7 @@ static void oak_setdisplaystart(int address)
 
 static void oak_setlogicalwidth(int width)
 {
-    outw(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
+    port_outw_r(CRT_IC, 0x13 + (width >> 3) * 256);	/* lw3-lw11 */
 }
 
 /* oak_init ( int force, int par1, int par2)            Initialize chipset
@@ -707,7 +707,7 @@ static void OAK_WriteDAC(int dac_value)
     port_in(DAC_SC11487);
     port_in(DAC_SC11487);
     port_in(DAC_SC11487);
-    outb(DAC_SC11487, dac_value);
+    port_out_r(DAC_SC11487, dac_value);
     port_in(DAC_REVERT);		/* and reset DAC ... */
 }
 
@@ -720,8 +720,8 @@ static void OAK_WriteDAC(int dac_value)
 
 static void OAKSetByte(int OTI_index, int OTI_value)
 {
-    outb(OTI_INDEX, OTI_index);
-    outb(OTI_R_W, OTI_value);
+    port_out_r(OTI_INDEX, OTI_index);
+    port_out_r(OTI_R_W, OTI_value);
 }
 
 /* OAKGetByte ( index )                                 Returns ext. register
@@ -732,7 +732,7 @@ static void OAKSetByte(int OTI_index, int OTI_value)
 
 static int OAKGetByte(int OTI_index)
 {
-    outb(OTI_INDEX, OTI_index);
+    port_out_r(OTI_INDEX, OTI_index);
     return port_in(OTI_R_W);
 }
 
