@@ -42,6 +42,8 @@ static struct
 
 static int SVGALIB_VESA[__GLASTMODE+1];
 
+static struct vbe_crtc_info_block *vesa_crtc;
+
 static void vesa_setpage(int page)
 {
 vesa_r.eax=0x4f05;
@@ -163,6 +165,18 @@ static int vesa_modeavailable(int mode)
 return SVGALIB_VESA[mode];
 }
 
+/* Set CRTC info related registers when setting video mode */
+static void vesa_set_crtc_info_regs( void )
+{
+	if (vesa_chiptype >= 2)
+	{
+		vesa_r.ebx |= 0x0800;
+		vesa_r.es = (unsigned int)vesa_crtc >> 4;
+		vesa_r.edi = (unsigned int)vesa_crtc & 0xf;
+	}
+}
+
+
 
 static int vesa_setmode(int mode, int prv_mode)
 {
@@ -180,6 +194,35 @@ static int vesa_setmode(int mode, int prv_mode)
     if (!vesa_modeavailable(mode)) return 1;
     vesa_r.eax=0x4f02;
     vesa_r.ebx=SVGALIB_VESA[mode]|0x8000|(vesa_is_linear*0x4000);
+
+	/* Set timing information */
+	if (vesa_chiptype >= 2)
+	{
+		ModeTiming *modetiming;
+		ModeInfo *modeinfo;
+
+		/* Get mode timing */
+    	modeinfo = __svgalib_createModeInfoStructureForSvgalibMode(mode);
+		modetiming = malloc(sizeof(ModeTiming));
+		if (!__svgalib_getmodetiming(modetiming, modeinfo, cardspecs))
+		{
+			/* Fill VBE Timing structure */
+			memset(vesa_crtc, 0, sizeof(*vesa_crtc));
+			vesa_crtc->horiz_start = modetiming->HSyncStart;
+			vesa_crtc->horiz_end = modetiming->HSyncEnd;
+			vesa_crtc->horiz_total = modetiming->HTotal;
+			vesa_crtc->vert_start = modetiming->VSyncStart;
+			vesa_crtc->vert_end = modetiming->VSyncEnd;
+			vesa_crtc->vert_total = modetiming->VTotal;
+			vesa_crtc->pixel_clock = modetiming->pixelClock * 1000;
+			vesa_crtc->refresh_rate = (unsigned short)
+				(100 * (vesa_crtc->pixel_clock / 
+				(vesa_crtc->vert_total * vesa_crtc->horiz_total)));
+		}
+		free(modetiming);
+		free(modeinfo);
+	}
+	vesa_set_crtc_info_regs();
     vesa_last_mode_set=vesa_r.ebx;
     __svgalib_LRMI_callbacks->rm_int(0x10,&vesa_r);
 
@@ -292,6 +335,7 @@ static int vesa_linear(int op, int param)
 if (op==LINEAR_ENABLE) {  
   vesa_r.eax=0x4f02;
   vesa_r.ebx=vesa_last_mode_set|0x4000;
+  vesa_set_crtc_info_regs();
   __svgalib_LRMI_callbacks->rm_int(0x10,&vesa_r);
   vesa_is_linear=1;
 };
@@ -299,6 +343,7 @@ if (op==LINEAR_ENABLE) {
 if (op==LINEAR_DISABLE){ 
   vesa_r.eax=0x4f02;
   vesa_r.ebx=vesa_last_mode_set;
+  vesa_set_crtc_info_regs();
   __svgalib_LRMI_callbacks->rm_int(0x10,&vesa_r);
   vesa_is_linear=0;
 };
