@@ -230,7 +230,7 @@ static char *kbd_process_option(int option, int mode, char **nptr) {
   store_event:
 
 #ifdef DEBUG_KEYBOARD
-  printf(" Fake Mouse Event: scancode: %ld;  type: %ld;  data: %ld;  flags: %ld\n", (long)scancode, (long)type, (long)data, (long)flags);
+  fprintf(stderr," Fake Mouse Event: scancode: %ld;  type: %ld;  data: %ld;  flags: %ld\n", (long)scancode, (long)type, (long)data, (long)flags);
 #endif
 	   if ( ! expand_events(n))
 	     return ptr;
@@ -336,7 +336,7 @@ int keyboard_init_return_fd(void) {
         __svgalib_kbd_fd = __svgalib_tty_fd; /* We are initialized. */
     
         if (ioctl(__svgalib_kbd_fd, KDGKBMODE, &oldkbmode)) {
-	    printf("svgalib: cannot get keyboard mode.\n");
+	    fprintf(stderr,"svgalib: cannot get keyboard mode.\n");
 	    return -1;
         }
         tcgetattr(__svgalib_kbd_fd, &oldkbdtermios);
@@ -527,15 +527,15 @@ static int keyboard_getevents(int wait)
                     flags = event -> flags;
                     request_at_down = (event -> flags & FMEF_AT_PRESS)  ?  1 : 0;
 #ifdef DEBUG_KEYBOARD
-                    printf(" event type: %ld;  flags: %ld;  data: %ld; is_pressed: %ld\n", (long)event->type, (long)flags, (long)event->data, (long)is_pressed);
+                    fprintf(stderr," event type: %ld;  flags: %ld;  data: %ld; is_pressed: %ld\n", (long)event->type, (long)flags, (long)event->data, (long)is_pressed);
 #endif
                     if (flags & FMEF_AT_BOTH  ||  request_at_down == is_pressed) {
 #ifdef DEBUG_KEYBOARD
-                        printf("   flags: %ld\n", (long)flags);
+                        fprintf(stderr,"   flags: %ld\n", (long)flags);
 #endif
                         if (  (! (flags & FMEF_TRIGGERED))  ||  flags & FMEF_REPEAT) {
 #ifdef DEBUG_KEYBOARD
-                            printf("      triggering\n");
+                            fprintf(stderr,"      triggering\n");
 #endif
                             switch (event -> type) {
                             case FME_TYPE_BUTTON1:
@@ -580,7 +580,7 @@ static int keyboard_getevents(int wait)
               __svgalib_mouse_eventhandler(but, dx, dy, dz, 0, 0, 0);
 
 #ifdef DEBUG_KEYBOARD
-            printf("\tfake_mouse_event triggered; but: %ld; dx: %ld; dy: %ld; dz: %ld\n",
+            fprintf(stderr,"\tfake_mouse_event triggered; but: %ld; dx: %ld; dy: %ld; dz: %ld\n",
                    (long)but, (long)dx, (long)dy, (long)dz);
 #endif
           }
@@ -652,7 +652,7 @@ void keyboard_setdefaulteventhandler(void)
 static int checkscancode(int scancode)
 {
     if (scancode < 0 || scancode >= NR_KEYS) {
-	printf("svgalib: keyboard scancode out of range (%d).\n",
+	fprintf(stderr,"svgalib: keyboard scancode out of range (%d).\n",
 	       scancode);
 	return 1;
     }
@@ -706,7 +706,7 @@ static void default_handler(int scancode, int newstate)
 
 #if 0				/* This happens very often. */
     if (state[scancode] == newstate) {
-	printf("svgalib: keyboard event does not match (scancode = %d)\n",
+	fprintf(stderr,"svgalib: keyboard event does not match (scancode = %d)\n",
 	       scancode);
 	return;
     }
