@@ -9,6 +9,8 @@
 #include "../libvga.h"
 
 extern int mouse_open;
+extern char *__svgalib_mouse_device;
+extern int __svgalib_mouse_flag;
 
 void (*__svgalib_mouse_eventhandler) (int, int, int, int, int, int, int) = NULL;
 
@@ -73,6 +75,11 @@ int mouse_init(char *dev, int type, int samplerate)
 {
     if (mouse_open)
         return 0;
+    if(dev==NULL) {
+        dev = __svgalib_mouse_device;
+        type = __svgalib_mouse_flag;
+        samplerate=MOUSE_DEFAULTSAMPLERATE;
+    }
     if (mouse_init_return_fd(dev, type, samplerate) == -1)
 	return -1;
     else {
@@ -339,7 +346,7 @@ void mouse_setrange_6d(int x1, int x2, int y1, int y2, int z1, int z2,
 void mouse_setscale(int s)
 {
     if (scale==0)
-      printf("Mouse scale must be non-zero!\n");
+      fprintf(stderr,"Mouse scale must be non-zero!\n");
     else {
       mouse_x = (mouse_x*s)/scale;
       mouse_y = (mouse_y*s)/scale;
