@@ -137,11 +137,11 @@ static int timing_within_monitor_spec(MonitorModeTiming * mmtp)
 	vsf *= 2.0f;
     if ((mmtp->flags & DOUBLESCAN))
 	vsf /= 2.0f;
-#ifdef DEBUG
-    fprintf(stderr,"hsf = %f (in:%d), vsf = %f (in:%d)\n",
+
+    DPRINTF("hsf = %f (in:%d), vsf = %f (in:%d)\n",
 	   hsf / 1000, (int) INRANGE(hsf, horizsync),
 	   vsf, (int) INRANGE(vsf, vertrefresh));
-#endif
+
     return INRANGE(hsf, horizsync) && INRANGE(vsf, vertrefresh);
 }
 
@@ -639,7 +639,7 @@ static GTF_constants GC = {
 static double round(double v)
 {
    	double u=v;
-        long j;
+        int j;
         
         if(u<0) u=-u;
         u=u+0.5;
