@@ -1,8 +1,40 @@
+#if NOHELPER
+#include <sys/io.h>
+void __svgalib_port_rep_outb(unsigned char* string, int length, int port)
+{
+	outsb(port, string, length);
+}
+void __svgalib_port_out(int value, int port)
+{
+	outb(value, port);
+}
+void __svgalib_port_outw(int value, int port)
+{
+	outw(value, port);
+}
+void __svgalib_port_outl(unsigned int value, int port)
+{
+	outl(value, port);
+}
+int __svgalib_port_in(int port)
+{
+	return inb(port);
+}
+int __svgalib_port_inw(int port)
+{
+	return inw(port);
+}
+int __svgalib_port_inl(int port)
+{
+	return inl(port);
+}
+
+#else
 #include <sys/ioctl.h>
 #include "svgalib_helper.h"
 #include "libvga.h"
 
-void port_rep_outb(unsigned char* string, int length, int port)
+void __svgalib_port_rep_outb(unsigned char* string, int length, int port)
 {
   io_string_t iostr;
 
@@ -13,7 +45,7 @@ void port_rep_outb(unsigned char* string, int length, int port)
   ioctl(__svgalib_mem_fd,SVGAHELPER_REPOUTB,&iostr); 
 }
 
-void port_out(int value, int port)
+void __svgalib_port_out(int value, int port)
 {
     io_t iov;
     
@@ -22,7 +54,7 @@ void port_out(int value, int port)
     ioctl(__svgalib_mem_fd,SVGAHELPER_OUTB,&iov);
 }
 
-void port_outw(int value, int port)
+void __svgalib_port_outw(int value, int port)
 {
     io_t iov;
     
@@ -31,7 +63,7 @@ void port_outw(int value, int port)
     ioctl(__svgalib_mem_fd,SVGAHELPER_OUTW,&iov);
 }
 
-void port_outl(int value, int port)
+void __svgalib_port_outl(int value, int port)
 {
     io_t iov;
     
@@ -40,7 +72,7 @@ void port_outl(int value, int port)
     ioctl(__svgalib_mem_fd,SVGAHELPER_OUTL,&iov);
 }
 
-int port_in(int port)
+int __svgalib_port_in(int port)
 {
     io_t iov;
     
@@ -50,7 +82,7 @@ int port_in(int port)
     return iov.val;
 }
 
-int port_inw(int port)
+int __svgalib_port_inw(int port)
 {
     io_t iov;
     
@@ -60,7 +92,7 @@ int port_inw(int port)
     return iov.val;
 }
 
-int port_inl(int port)
+int __svgalib_port_inl(int port)
 {
     io_t iov;
     
@@ -69,4 +101,4 @@ int port_inl(int port)
 
     return iov.val;
 }
-
+#endif
