@@ -145,7 +145,7 @@ int vga_drawscansegment(unsigned char *colors, int x, int y, int length)
 		if (last > length)
 		    last = length;
 		for (j = first; j < last; j++, i++)
-		    bytes.i = (bytes.i << 1) | color16[colors[j]].i;
+		    bytes.i = (bytes.i << 1) | color16[colors[j]&15].i;
 		plane0[k] = bytes.b.bit0;
 		plane1[k] = bytes.b.bit1;
 		plane2[k] = bytes.b.bit2;
