@@ -28,10 +28,12 @@ vga_cardinfo *vga_getcardinfo(void) {
     
     vci = malloc(sizeof(vga_cardinfo));
     if(vci==NULL) return vci;
-    vci->version = 0x100;
+    vci->version = 0x200;
     vci->size = sizeof(vga_cardinfo);
     vci->chipset = __svgalib_chipset;
     vci->physmem = __svgalib_linear_mem_base;
+	vci->physmemsize = __svgalib_linear_mem_size;
+	vci->linearmem = LINEAR_POINTER;
     
     return vci;
 }
